@@ -11,7 +11,7 @@
 *************************************************************************
 ** A TCL Interface to SQLite
 **
-** $Id: tclsqlite.c,v 1.39 2002/07/15 20:58:48 drh Exp $
+** $Id: tclsqlite.c,v 1.48 2003/04/23 12:25:24 drh Exp $
 */
 #ifndef NO_TCL     /* Omit this whole file if TCL is unavailable */
 
@@ -31,6 +31,17 @@
 # define UTF_TRANSLATION_NEEDED 1
 #endif
 
+/*
+** New SQL functions can be created as TCL scripts.  Each such function
+** is described by an instance of the following structure.
+*/
+typedef struct SqlFunc SqlFunc;
+struct SqlFunc {
+  Tcl_Interp *interp;   /* The TCL interpret to execute the function */
+  char *zScript;        /* The script to be run */
+  SqlFunc *pNext;       /* Next function on the list of them all */
+};
+
 /*
 ** There is one instance of this structure for each SQLite database
 ** that has been opened by the SQLite TCL interface.
@@ -40,6 +51,10 @@ struct SqliteDb {
   sqlite *db;           /* The "real" database structure */
   Tcl_Interp *interp;   /* The interpreter used for this database */
   char *zBusy;          /* The busy callback routine */
+  char *zTrace;         /* The trace callback routine */
+  char *zAuth;          /* The authorization callback routine */
+  SqlFunc *pFunc;       /* List of SQL functions */
+  int rc;               /* Return code of most recent sqlite_exec() */
 };
 
 /*
@@ -239,9 +254,20 @@ static int DbEvalCallback2(
 static void DbDeleteCmd(void *db){
   SqliteDb *pDb = (SqliteDb*)db;
   sqlite_close(pDb->db);
+  while( pDb->pFunc ){
+    SqlFunc *pFunc = pDb->pFunc;
+    pDb->pFunc = pFunc->pNext;
+    Tcl_Free((char*)pFunc);
+  }
   if( pDb->zBusy ){
     Tcl_Free(pDb->zBusy);
   }
+  if( pDb->zTrace ){
+    Tcl_Free(pDb->zTrace);
+  }
+  if( pDb->zAuth ){
+    Tcl_Free(pDb->zAuth);
+  }
   Tcl_Free((char*)pDb);
 }
 
@@ -270,6 +296,115 @@ static int DbBusyHandler(void *cd, const char *zTable, int nTries){
   return 1;
 }
 
+/*
+** This routine is called by the SQLite trace handler whenever a new
+** block of SQL is executed.  The TCL script in pDb->zTrace is executed.
+*/
+static void DbTraceHandler(void *cd, const char *zSql){
+  SqliteDb *pDb = (SqliteDb*)cd;
+  Tcl_DString str;
+
+  Tcl_DStringInit(&str);
+  Tcl_DStringAppend(&str, pDb->zTrace, -1);
+  Tcl_DStringAppendElement(&str, zSql);
+  Tcl_Eval(pDb->interp, Tcl_DStringValue(&str));
+  Tcl_DStringFree(&str);
+  Tcl_ResetResult(pDb->interp);
+}
+
+/*
+** This routine is called to evaluate an SQL function implemented
+** using TCL script.
+*/
+static void tclSqlFunc(sqlite_func *context, int argc, const char **argv){
+  SqlFunc *p = sqlite_user_data(context);
+  Tcl_DString cmd;
+  int i;
+  int rc;
+
+  Tcl_DStringInit(&cmd);
+  Tcl_DStringAppend(&cmd, p->zScript, -1);
+  for(i=0; i<argc; i++){
+    Tcl_DStringAppendElement(&cmd, argv[i] ? argv[i] : "");
+  }
+  rc = Tcl_Eval(p->interp, Tcl_DStringValue(&cmd));
+  if( rc ){
+    sqlite_set_result_error(context, Tcl_GetStringResult(p->interp), -1); 
+  }else{
+    sqlite_set_result_string(context, Tcl_GetStringResult(p->interp), -1);
+  }
+}
+#ifndef SQLITE_OMIT_AUTHORIZATION
+/*
+** This is the authentication function.  It appends the authentication
+** type code and the two arguments to zCmd[] then invokes the result
+** on the interpreter.  The reply is examined to determine if the
+** authentication fails or succeeds.
+*/
+static int auth_callback(
+  void *pArg,
+  int code,
+  const char *zArg1,
+  const char *zArg2,
+  const char *zArg3,
+  const char *zArg4
+){
+  char *zCode;
+  Tcl_DString str;
+  int rc;
+  const char *zReply;
+  SqliteDb *pDb = (SqliteDb*)pArg;
+
+  switch( code ){
+    case SQLITE_COPY              : zCode="SQLITE_COPY"; break;
+    case SQLITE_CREATE_INDEX      : zCode="SQLITE_CREATE_INDEX"; break;
+    case SQLITE_CREATE_TABLE      : zCode="SQLITE_CREATE_TABLE"; break;
+    case SQLITE_CREATE_TEMP_INDEX : zCode="SQLITE_CREATE_TEMP_INDEX"; break;
+    case SQLITE_CREATE_TEMP_TABLE : zCode="SQLITE_CREATE_TEMP_TABLE"; break;
+    case SQLITE_CREATE_TEMP_TRIGGER: zCode="SQLITE_CREATE_TEMP_TRIGGER"; break;
+    case SQLITE_CREATE_TEMP_VIEW  : zCode="SQLITE_CREATE_TEMP_VIEW"; break;
+    case SQLITE_CREATE_TRIGGER    : zCode="SQLITE_CREATE_TRIGGER"; break;
+    case SQLITE_CREATE_VIEW       : zCode="SQLITE_CREATE_VIEW"; break;
+    case SQLITE_DELETE            : zCode="SQLITE_DELETE"; break;
+    case SQLITE_DROP_INDEX        : zCode="SQLITE_DROP_INDEX"; break;
+    case SQLITE_DROP_TABLE        : zCode="SQLITE_DROP_TABLE"; break;
+    case SQLITE_DROP_TEMP_INDEX   : zCode="SQLITE_DROP_TEMP_INDEX"; break;
+    case SQLITE_DROP_TEMP_TABLE   : zCode="SQLITE_DROP_TEMP_TABLE"; break;
+    case SQLITE_DROP_TEMP_TRIGGER : zCode="SQLITE_DROP_TEMP_TRIGGER"; break;
+    case SQLITE_DROP_TEMP_VIEW    : zCode="SQLITE_DROP_TEMP_VIEW"; break;
+    case SQLITE_DROP_TRIGGER      : zCode="SQLITE_DROP_TRIGGER"; break;
+    case SQLITE_DROP_VIEW         : zCode="SQLITE_DROP_VIEW"; break;
+    case SQLITE_INSERT            : zCode="SQLITE_INSERT"; break;
+    case SQLITE_PRAGMA            : zCode="SQLITE_PRAGMA"; break;
+    case SQLITE_READ              : zCode="SQLITE_READ"; break;
+    case SQLITE_SELECT            : zCode="SQLITE_SELECT"; break;
+    case SQLITE_TRANSACTION       : zCode="SQLITE_TRANSACTION"; break;
+    case SQLITE_UPDATE            : zCode="SQLITE_UPDATE"; break;
+    default                       : zCode="????"; break;
+  }
+  Tcl_DStringInit(&str);
+  Tcl_DStringAppend(&str, pDb->zAuth, -1);
+  Tcl_DStringAppendElement(&str, zCode);
+  Tcl_DStringAppendElement(&str, zArg1 ? zArg1 : "");
+  Tcl_DStringAppendElement(&str, zArg2 ? zArg2 : "");
+  Tcl_DStringAppendElement(&str, zArg3 ? zArg3 : "");
+  Tcl_DStringAppendElement(&str, zArg4 ? zArg4 : "");
+  rc = Tcl_GlobalEval(pDb->interp, Tcl_DStringValue(&str));
+  Tcl_DStringFree(&str);
+  zReply = Tcl_GetStringResult(pDb->interp);
+  if( strcmp(zReply,"SQLITE_OK")==0 ){
+    rc = SQLITE_OK;
+  }else if( strcmp(zReply,"SQLITE_DENY")==0 ){
+    rc = SQLITE_DENY;
+  }else if( strcmp(zReply,"SQLITE_IGNORE")==0 ){
+    rc = SQLITE_IGNORE;
+  }else{
+    rc = 999;
+  }
+  return rc;
+}
+#endif /* SQLITE_OMIT_AUTHORIZATION */
+
 /*
 ** The "sqlite" command below creates a new Tcl command for each
 ** connection it opens to an SQLite database.  This routine is invoked
@@ -287,14 +422,16 @@ static int DbObjCmd(void *cd, Tcl_Interp *interp, int objc,Tcl_Obj *const*objv){
   SqliteDb *pDb = (SqliteDb*)cd;
   int choice;
   static const char *DB_strs[] = {
-    "busy",               "changes",           "close",
-    "complete",           "eval",              "last_insert_rowid",
-    "open_aux_file",      "timeout",           0
+    "authorizer",         "busy",              "changes",
+    "close",              "complete",          "errorcode",
+    "eval",               "function",          "last_insert_rowid",
+    "timeout",            "trace",             0
   };
   enum DB_enum {
-    DB_BUSY,              DB_CHANGES,          DB_CLOSE,
-    DB_COMPLETE,          DB_EVAL,             DB_LAST_INSERT_ROWID,
-    DB_OPEN_AUX_FILE,     DB_TIMEOUT,          
+    DB_AUTHORIZER,        DB_BUSY,             DB_CHANGES,
+    DB_CLOSE,             DB_COMPLETE,         DB_ERRORCODE,
+    DB_EVAL,              DB_FUNCTION,         DB_LAST_INSERT_ROWID,
+    DB_TIMEOUT,           DB_TRACE,            
   };
 
   if( objc<2 ){
@@ -307,6 +444,56 @@ static int DbObjCmd(void *cd, Tcl_Interp *interp, int objc,Tcl_Obj *const*objv){
 
   switch( (enum DB_enum)choice ){
 
+  /*    $db authorizer ?CALLBACK?
+  **
+  ** Invoke the given callback to authorize each SQL operation as it is
+  ** compiled.  5 arguments are appended to the callback before it is
+  ** invoked:
+  **
+  **   (1) The authorization type (ex: SQLITE_CREATE_TABLE, SQLITE_INSERT, ...)
+  **   (2) First descriptive name (depends on authorization type)
+  **   (3) Second descriptive name
+  **   (4) Name of the database (ex: "main", "temp")
+  **   (5) Name of trigger that is doing the access
+  **
+  ** The callback should return on of the following strings: SQLITE_OK,
+  ** SQLITE_IGNORE, or SQLITE_DENY.  Any other return value is an error.
+  **
+  ** If this method is invoked with no arguments, the current authorization
+  ** callback string is returned.
+  */
+  case DB_AUTHORIZER: {
+    if( objc>3 ){
+      Tcl_WrongNumArgs(interp, 2, objv, "?CALLBACK?");
+    }else if( objc==2 ){
+      if( pDb->zAuth ){
+        Tcl_AppendResult(interp, pDb->zAuth, 0);
+      }
+    }else{
+      char *zAuth;
+      int len;
+      if( pDb->zAuth ){
+        Tcl_Free(pDb->zAuth);
+      }
+      zAuth = Tcl_GetStringFromObj(objv[2], &len);
+      if( zAuth && len>0 ){
+        pDb->zAuth = Tcl_Alloc( len + 1 );
+        strcpy(pDb->zAuth, zAuth);
+      }else{
+        pDb->zAuth = 0;
+      }
+#ifndef SQLITE_OMIT_AUTHORIZATION
+      if( pDb->zAuth ){
+        pDb->interp = interp;
+        sqlite_set_authorizer(pDb->db, auth_callback, pDb);
+      }else{
+        sqlite_set_authorizer(pDb->db, 0, 0);
+      }
+#endif
+    }
+    break;
+  }
+
   /*    $db busy ?CALLBACK?
   **
   ** Invoke the given callback if an SQL statement attempts to open
@@ -389,6 +576,17 @@ static int DbObjCmd(void *cd, Tcl_Interp *interp, int objc,Tcl_Obj *const*objv){
     Tcl_SetBooleanObj(pResult, isComplete);
     break;
   }
+
+  /*
+  **    $db errorcode
+  **
+  ** Return the numeric error code that was returned by the most recent
+  ** call to sqlite_exec().
+  */
+  case DB_ERRORCODE: {
+    Tcl_SetObjResult(interp, Tcl_NewIntObj(pDb->rc));
+    break;
+  }
    
   /*
   **    $db eval $sql ?array {  ...code... }?
@@ -442,15 +640,18 @@ static int DbObjCmd(void *cd, Tcl_Interp *interp, int objc,Tcl_Obj *const*objv){
       rc = sqlite_exec(pDb->db, zSql, DbEvalCallback2, pList, &zErrMsg);
       Tcl_SetObjResult(interp, pList);
     }
-    if( zErrMsg ){
+    pDb->rc = rc;
+    if( rc==SQLITE_ABORT ){
+      if( zErrMsg ) free(zErrMsg);
+      rc = cbData.tcl_rc;
+    }else if( zErrMsg ){
       Tcl_SetResult(interp, zErrMsg, TCL_VOLATILE);
       free(zErrMsg);
       rc = TCL_ERROR;
-    }else if( rc!=SQLITE_OK && rc!=SQLITE_ABORT ){
+    }else if( rc!=SQLITE_OK ){
       Tcl_AppendResult(interp, sqlite_error_string(rc), 0);
       rc = TCL_ERROR;
     }else{
-      rc = cbData.tcl_rc;
     }
     Tcl_DecrRefCount(objv[2]);
 #ifdef UTF_TRANSLATION_NEEDED
@@ -466,6 +667,34 @@ static int DbObjCmd(void *cd, Tcl_Interp *interp, int objc,Tcl_Obj *const*objv){
     return rc;
   }
 
+  /*
+  **     $db function NAME SCRIPT
+  **
+  ** Create a new SQL function called NAME.  Whenever that function is
+  ** called, invoke SCRIPT to evaluate the function.
+  */
+  case DB_FUNCTION: {
+    SqlFunc *pFunc;
+    char *zName;
+    char *zScript;
+    int nScript;
+    if( objc!=4 ){
+      Tcl_WrongNumArgs(interp, 2, objv, "NAME SCRIPT");
+      return TCL_ERROR;
+    }
+    zName = Tcl_GetStringFromObj(objv[2], 0);
+    zScript = Tcl_GetStringFromObj(objv[3], &nScript);
+    pFunc = (SqlFunc*)Tcl_Alloc( sizeof(*pFunc) + nScript + 1 );
+    if( pFunc==0 ) return TCL_ERROR;
+    pFunc->interp = interp;
+    pFunc->pNext = pDb->pFunc;
+    pFunc->zScript = (char*)&pFunc[1];
+    strcpy(pFunc->zScript, zScript);
+    sqlite_create_function(pDb->db, zName, -1, tclSqlFunc, pFunc);
+    sqlite_function_type(pDb->db, zName, SQLITE_NUMERIC);
+    break;
+  }
+
   /*
   **     $db last_insert_rowid 
   **
@@ -484,34 +713,6 @@ static int DbObjCmd(void *cd, Tcl_Interp *interp, int objc,Tcl_Obj *const*objv){
     break;
   }
 
-  /*
-  **     $db open_aux_file  FILENAME
-  **
-  ** Begin using FILENAME as the database file used to store temporary
-  ** tables.
-  */
-  case DB_OPEN_AUX_FILE: {
-    const char *zFilename;
-    char *zErrMsg = 0;
-    int rc;
-    if( objc!=3 ){
-      Tcl_WrongNumArgs(interp, 2, objv, "FILENAME");
-      return TCL_ERROR;
-    }
-    zFilename = Tcl_GetStringFromObj(objv[2], 0);
-    rc = sqlite_open_aux_file(pDb->db, zFilename, &zErrMsg);
-    if( rc!=0 ){
-      if( zErrMsg ){
-        Tcl_AppendResult(interp, zErrMsg, 0);
-        free(zErrMsg);
-      }else{
-        Tcl_AppendResult(interp, sqlite_error_string(rc), 0);
-      }
-      return TCL_ERROR;
-    }
-    break;
-  }
-
   /*
   **     $db timeout MILLESECONDS
   **
@@ -527,6 +728,43 @@ static int DbObjCmd(void *cd, Tcl_Interp *interp, int objc,Tcl_Obj *const*objv){
     sqlite_busy_timeout(pDb->db, ms);
     break;
   }
+
+  /*    $db trace ?CALLBACK?
+  **
+  ** Make arrangements to invoke the CALLBACK routine for each SQL statement
+  ** that is executed.  The text of the SQL is appended to CALLBACK before
+  ** it is executed.
+  */
+  case DB_TRACE: {
+    if( objc>3 ){
+      Tcl_WrongNumArgs(interp, 2, objv, "?CALLBACK?");
+    }else if( objc==2 ){
+      if( pDb->zTrace ){
+        Tcl_AppendResult(interp, pDb->zTrace, 0);
+      }
+    }else{
+      char *zTrace;
+      int len;
+      if( pDb->zTrace ){
+        Tcl_Free(pDb->zTrace);
+      }
+      zTrace = Tcl_GetStringFromObj(objv[2], &len);
+      if( zTrace && len>0 ){
+        pDb->zTrace = Tcl_Alloc( len + 1 );
+        strcpy(pDb->zTrace, zTrace);
+      }else{
+        pDb->zTrace = 0;
+      }
+      if( pDb->zTrace ){
+        pDb->interp = interp;
+        sqlite_trace(pDb->db, DbTraceHandler, pDb);
+      }else{
+        sqlite_trace(pDb->db, 0, 0);
+      }
+    }
+    break;
+  }
+
   } /* End of the SWITCH statement */
   return TCL_OK;
 }
@@ -552,6 +790,10 @@ static int DbObjCmd(void *cd, Tcl_Interp *interp, int objc,Tcl_Obj *const*objv){
 **       Return the encoding used by LIKE and GLOB operators.  Choices
 **       are UTF-8 and iso8859.
 **
+**  sqlite -version
+**
+**       Return the version number of the SQLite library.
+**
 **  sqlite -tcl-uses-utf
 **
 **       Return "1" if compiled with a Tcl uses UTF-8.  Return "0" if
@@ -568,6 +810,10 @@ static int DbMain(void *cd, Tcl_Interp *interp, int argc, char **argv){
       Tcl_AppendResult(interp,sqlite_encoding,0);
       return TCL_OK;
     }
+    if( strcmp(argv[1],"-version")==0 ){
+      Tcl_AppendResult(interp,sqlite_version,0);
+      return TCL_OK;
+    }
     if( strcmp(argv[1],"-tcl-uses-utf")==0 ){
 #ifdef TCL_UTF_MAX
       Tcl_AppendResult(interp,"1",0);
@@ -643,13 +889,13 @@ static int DbMain(void *cd, Tcl_Interp *interp, int argc, char **argv){
 */
 int Sqlite_Init(Tcl_Interp *interp){
   Tcl_InitStubs(interp, "8.0", 0);
-  Tcl_CreateCommand(interp, "sqlite", DbMain, 0, 0);
+  Tcl_CreateCommand(interp, "sqlite", (Tcl_CmdProc*)DbMain, 0, 0);
   Tcl_PkgProvide(interp, "sqlite", "2.0");
   return TCL_OK;
 }
 int Tclsqlite_Init(Tcl_Interp *interp){
   Tcl_InitStubs(interp, "8.0", 0);
-  Tcl_CreateCommand(interp, "sqlite", DbMain, 0, 0);
+  Tcl_CreateCommand(interp, "sqlite", (Tcl_CmdProc*)DbMain, 0, 0);
   Tcl_PkgProvide(interp, "sqlite", "2.0");
   return TCL_OK;
 }
