@@ -13,13 +13,72 @@
 ** is not included in the SQLite library.  It is used for automated
 ** testing of the SQLite library.
 **
-** $Id: test1.c,v 1.12 2002/07/10 21:26:01 drh Exp $
+** $Id: test1.c,v 1.24 2003/04/26 13:19:39 drh Exp $
 */
 #include "sqliteInt.h"
 #include "tcl.h"
+#include "os.h"
 #include <stdlib.h>
 #include <string.h>
 
+#if OS_WIN
+# define PTR_FMT "%x"
+#else
+# define PTR_FMT "%p"
+#endif
+
+/*
+** Decode a pointer to an sqlite object.
+*/
+static int getDbPointer(Tcl_Interp *interp, const char *zArg, sqlite **ppDb){
+  if( sscanf(zArg, PTR_FMT, (void**)ppDb)!=1 ){
+    Tcl_AppendResult(interp, "\"", zArg, "\" is not a valid pointer value", 0);
+    return TCL_ERROR;
+  }
+  return TCL_OK;
+}
+
+/*
+** Decode a pointer to an sqlite_vm object.
+*/
+static int getVmPointer(Tcl_Interp *interp, const char *zArg, sqlite_vm **ppVm){
+  if( sscanf(zArg, PTR_FMT, (void**)ppVm)!=1 ){
+    Tcl_AppendResult(interp, "\"", zArg, "\" is not a valid pointer value", 0);
+    return TCL_ERROR;
+  }
+  return TCL_OK;
+}
+
+/*
+** Generate a text representation of a pointer that can be understood
+** by the getDbPointer and getVmPointer routines above.
+**
+** The problem is, on some machines (Solaris) if you do a printf with
+** "%p" you cannot turn around and do a scanf with the same "%p" and
+** get your pointer back.  You have to prepend a "0x" before it will
+** work.  Or at least that is what is reported to me (drh).  But this
+** behavior varies from machine to machine.  The solution used her is
+** to test the string right after it is generated to see if it can be
+** understood by scanf, and if not, try prepending an "0x" to see if
+** that helps.  If nothing works, a fatal error is generated.
+*/
+static int makePointerStr(Tcl_Interp *interp, char *zPtr, void *p){
+  void *p2;
+  sprintf(zPtr, PTR_FMT, p);
+  if( sscanf(zPtr, PTR_FMT, &p2)!=1 || p2!=p ){
+    sprintf(zPtr, "0x" PTR_FMT, p);
+    if( sscanf(zPtr, PTR_FMT, &p2)!=1 || p2!=p ){
+      Tcl_AppendResult(interp, "unable to convert a pointer to a string "
+         "in the file " __FILE__ " in function makePointerStr().  Please "
+         "report this problem to the SQLite mailing list or as a new but "
+         "report.  Please provide detailed information about how you compiled "
+         "SQLite and what computer you are running on.", 0);
+      return TCL_ERROR;
+    }
+  }
+  return TCL_OK;
+}
+
 /*
 ** Usage:   sqlite_open filename
 **
@@ -45,7 +104,7 @@ static int sqlite_test_open(
     free(zErr);
     return TCL_ERROR;
   }
-  sprintf(zBuf,"%d",(int)db);
+  if( makePointerStr(interp, zBuf, db) ) return TCL_ERROR;
   Tcl_AppendResult(interp, zBuf, 0);
   return TCL_OK;
 }
@@ -91,7 +150,7 @@ static int test_exec_printf(
        " DB FORMAT STRING", 0);
     return TCL_ERROR;
   }
-  db = (sqlite*)strtol(argv[1], 0, 0);
+  if( getDbPointer(interp, argv[1], &db) ) return TCL_ERROR;
   Tcl_DStringInit(&str);
   rc = sqlite_exec_printf(db, argv[2], exec_printf_cb, &str, &zErr, argv[3]);
   sprintf(zBuf, "%d", rc);
@@ -128,7 +187,7 @@ static int test_get_table_printf(
        " DB FORMAT STRING", 0);
     return TCL_ERROR;
   }
-  db = (sqlite*)strtol(argv[1], 0, 0);
+  if( getDbPointer(interp, argv[1], &db) ) return TCL_ERROR;
   Tcl_DStringInit(&str);
   rc = sqlite_get_table_printf(db, argv[2], &aResult, &nRow, &nCol, 
                &zErr, argv[3]);
@@ -169,7 +228,7 @@ static int test_last_rowid(
     Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0], " DB\"", 0);
     return TCL_ERROR;
   }
-  db = (sqlite*)strtol(argv[1], 0, 0);
+  if( getDbPointer(interp, argv[1], &db) ) return TCL_ERROR;
   sprintf(zBuf, "%d", sqlite_last_insert_rowid(db));
   Tcl_AppendResult(interp, zBuf, 0);
   return SQLITE_OK;
@@ -192,7 +251,7 @@ static int sqlite_test_close(
        " FILENAME\"", 0);
     return TCL_ERROR;
   }
-  db = (sqlite*)strtol(argv[1], 0, 0);
+  if( getDbPointer(interp, argv[1], &db) ) return TCL_ERROR;
   sqlite_close(db);
   return TCL_OK;
 }
@@ -238,7 +297,7 @@ static void sqliteExecFunc(sqlite_func *context, int argc, const char **argv){
 ** sqlite_create_function function while a query is in progress in order
 ** to test the SQLITE_MISUSE detection logic.
 */
-static int sqlite_test_create_function(
+static int test_create_function(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
@@ -251,7 +310,7 @@ static int sqlite_test_create_function(
        " FILENAME\"", 0);
     return TCL_ERROR;
   }
-  db = (sqlite*)strtol(argv[1], 0, 0);
+  if( getDbPointer(interp, argv[1], &db) ) return TCL_ERROR;
   sqlite_create_function(db, "x_coalesce", -1, ifnullFunc, 0);
   sqlite_create_function(db, "x_sqlite_exec", 1, sqliteExecFunc, db);
   return TCL_OK;
@@ -288,7 +347,7 @@ static void countFinalize(sqlite_func *context){
 ** sqlite_create_aggregate function while a query is in progress in order
 ** to test the SQLITE_MISUSE detection logic.
 */
-static int sqlite_test_create_aggregate(
+static int test_create_aggregate(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
@@ -300,7 +359,7 @@ static int sqlite_test_create_aggregate(
        " FILENAME\"", 0);
     return TCL_ERROR;
   }
-  db = (sqlite*)strtol(argv[1], 0, 0);
+  if( getDbPointer(interp, argv[1], &db) ) return TCL_ERROR;
   sqlite_create_aggregate(db, "x_count", 0, countStep, countFinalize, 0);
   sqlite_create_aggregate(db, "x_count", 1, countStep, countFinalize, 0);
   return TCL_OK;
@@ -484,7 +543,7 @@ static void testFunc(sqlite_func *context, int argc, const char **argv){
 **
 ** Register the test SQL function on the database DB under the name NAME.
 */
-static int sqlite_register_test_function(
+static int test_register_func(
   void *NotUsed,
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int argc,              /* Number of arguments */
@@ -497,7 +556,7 @@ static int sqlite_register_test_function(
        " DB FUNCTION-NAME", 0);
     return TCL_ERROR;
   }
-  db = (sqlite*)strtol(argv[1], 0, 0);
+  if( getDbPointer(interp, argv[1], &db) ) return TCL_ERROR;
   rc = sqlite_create_function(db, argv[2], -1, testFunc, 0);
   if( rc!=0 ){
     Tcl_AppendResult(interp, sqlite_error_string(rc), 0);
@@ -506,32 +565,247 @@ static int sqlite_register_test_function(
   return TCL_OK;
 }
 
+/*
+** This SQLite callback records the datatype of all columns.
+**
+** The pArg argument is really a pointer to a TCL interpreter.  The
+** column names are inserted as the result of this interpreter.
+**
+** This routine returns non-zero which causes the query to abort.
+*/
+static int rememberDataTypes(void *pArg, int nCol, char **argv, char **colv){
+  int i;
+  Tcl_Interp *interp = (Tcl_Interp*)pArg;
+  Tcl_Obj *pList, *pElem;
+  if( colv[nCol+1]==0 ){
+    return 1;
+  }
+  pList = Tcl_NewObj();
+  for(i=0; i<nCol; i++){
+    pElem = Tcl_NewStringObj(colv[i+nCol] ? colv[i+nCol] : "NULL", -1);
+    Tcl_ListObjAppendElement(interp, pList, pElem);
+  }
+  Tcl_SetObjResult(interp, pList);
+  return 1;
+}
+
+/*
+** Invoke an SQL statement but ignore all the data in the result.  Instead,
+** return a list that consists of the datatypes of the various columns.
+**
+** This only works if "PRAGMA show_datatypes=on" has been executed against
+** the database connection.
+*/
+static int sqlite_datatypes(
+  void *NotUsed,
+  Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
+  int argc,              /* Number of arguments */
+  char **argv            /* Text of each argument */
+){
+  sqlite *db;
+  int rc;
+  if( argc!=3 ){
+    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0], 
+       " DB SQL", 0);
+    return TCL_ERROR;
+  }
+  if( getDbPointer(interp, argv[1], &db) ) return TCL_ERROR;
+  rc = sqlite_exec(db, argv[2], rememberDataTypes, interp, 0);
+  if( rc!=0 && rc!=SQLITE_ABORT ){
+    Tcl_AppendResult(interp, sqlite_error_string(rc), 0);
+    return TCL_ERROR;
+  }
+  return TCL_OK;
+}
+
+/*
+** Usage:  sqlite_compile  DB  SQL  TAILVAR
+**
+** Attempt to compile an SQL statement.  Return a pointer to the virtual
+** machine used to execute that statement.  Unprocessed SQL is written
+** into TAILVAR.
+*/
+static int test_compile(
+  void *NotUsed,
+  Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
+  int argc,              /* Number of arguments */
+  char **argv            /* Text of each argument */
+){
+  sqlite *db;
+  sqlite_vm *vm;
+  int rc;
+  char *zErr = 0;
+  const char *zTail;
+  char zBuf[50];
+  if( argc!=4 ){
+    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0], 
+       " DB SQL TAILVAR", 0);
+    return TCL_ERROR;
+  }
+  if( getDbPointer(interp, argv[1], &db) ) return TCL_ERROR;
+  rc = sqlite_compile(db, argv[2], &zTail, &vm, &zErr);
+  Tcl_SetVar(interp, argv[3], zTail, 0);
+  if( rc ){
+    assert( vm==0 );
+    sprintf(zBuf, "(%d) ", rc);
+    Tcl_AppendResult(interp, zBuf, zErr, 0);
+    sqlite_freemem(zErr);
+    return TCL_ERROR;
+  }
+  if( vm ){
+    if( makePointerStr(interp, zBuf, vm) ) return TCL_ERROR;
+    Tcl_AppendResult(interp, zBuf, 0);
+  }
+  return TCL_OK;
+}
+
+/*
+** Usage:  sqlite_step  VM  NVAR  VALUEVAR  COLNAMEVAR
+**
+** Step a virtual machine.  Return a the result code as a string.
+** Column results are written into three variables.
+*/
+static int test_step(
+  void *NotUsed,
+  Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
+  int argc,              /* Number of arguments */
+  char **argv            /* Text of each argument */
+){
+  sqlite_vm *vm;
+  int rc, i;
+  const char **azValue = 0;
+  const char **azColName = 0;
+  int N = 0;
+  char *zRc;
+  char zBuf[50];
+  if( argc!=5 ){
+    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0], 
+       " VM NVAR VALUEVAR COLNAMEVAR", 0);
+    return TCL_ERROR;
+  }
+  if( getVmPointer(interp, argv[1], &vm) ) return TCL_ERROR;
+  rc = sqlite_step(vm, &N, &azValue, &azColName);
+  sprintf(zBuf, "%d", N);
+  Tcl_SetVar(interp, argv[2], zBuf, 0);
+  Tcl_SetVar(interp, argv[3], "", 0);
+  if( azValue ){
+    for(i=0; i<N; i++){
+      Tcl_SetVar(interp, argv[3], azValue[i] ? azValue[i] : "",
+          TCL_APPEND_VALUE | TCL_LIST_ELEMENT);
+    }
+  }
+  Tcl_SetVar(interp, argv[4], "", 0);
+  if( azColName ){
+    for(i=0; i<N*2; i++){
+      Tcl_SetVar(interp, argv[4], azColName[i] ? azColName[i] : "",
+          TCL_APPEND_VALUE | TCL_LIST_ELEMENT);
+    }
+  }
+  switch( rc ){
+    case SQLITE_DONE:   zRc = "SQLITE_DONE";    break;
+    case SQLITE_BUSY:   zRc = "SQLITE_BUSY";    break;
+    case SQLITE_ROW:    zRc = "SQLITE_ROW";     break;
+    case SQLITE_ERROR:  zRc = "SQLITE_ERROR";   break;
+    case SQLITE_MISUSE: zRc = "SQLITE_MISUSE";  break;
+    default:            zRc = "unknown";        break;
+  }
+  Tcl_AppendResult(interp, zRc, 0);
+  return TCL_OK;
+}
+
+/*
+** Usage:  sqlite_finalize  VM 
+**
+** Shutdown a virtual machine.
+*/
+static int test_finalize(
+  void *NotUsed,
+  Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
+  int argc,              /* Number of arguments */
+  char **argv            /* Text of each argument */
+){
+  sqlite_vm *vm;
+  int rc;
+  char *zErrMsg = 0;
+  if( argc!=2 ){
+    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0], 
+       " VM\"", 0);
+    return TCL_ERROR;
+  }
+  if( getVmPointer(interp, argv[1], &vm) ) return TCL_ERROR;
+  rc = sqlite_finalize(vm, &zErrMsg);
+  if( rc ){
+    char zBuf[50];
+    sprintf(zBuf, "(%d) ", rc);
+    Tcl_AppendResult(interp, zBuf, zErrMsg, 0);
+    sqlite_freemem(zErrMsg);
+    return TCL_ERROR;
+  }
+  return TCL_OK;
+}
+
+/*
+** Usage:    breakpoint
+**
+** This routine exists for one purpose - to provide a place to put a
+** breakpoint with GDB that can be triggered using TCL code.  The use
+** for this is when a particular test fails on (say) the 1485th iteration.
+** In the TCL test script, we can add code like this:
+**
+**     if {$i==1485} breakpoint
+**
+** Then run testfixture in the debugger and wait for the breakpoint to
+** fire.  Then additional breakpoints can be set to trace down the bug.
+*/
+static int test_breakpoint(
+  void *NotUsed,
+  Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
+  int argc,              /* Number of arguments */
+  char **argv            /* Text of each argument */
+){
+  return TCL_OK;         /* Do nothing */
+}
+
 /*
 ** Register commands with the TCL interpreter.
 */
 int Sqlitetest1_Init(Tcl_Interp *interp){
   extern int sqlite_search_count;
-  Tcl_CreateCommand(interp, "sqlite_mprintf_int", sqlite_mprintf_int, 0, 0);
-  Tcl_CreateCommand(interp, "sqlite_mprintf_str", sqlite_mprintf_str, 0, 0);
-  Tcl_CreateCommand(interp, "sqlite_mprintf_double", sqlite_mprintf_double,0,0);
-  Tcl_CreateCommand(interp, "sqlite_open", sqlite_test_open, 0, 0);
-  Tcl_CreateCommand(interp, "sqlite_last_insert_rowid", test_last_rowid, 0, 0);
-  Tcl_CreateCommand(interp, "sqlite_exec_printf", test_exec_printf, 0, 0);
-  Tcl_CreateCommand(interp, "sqlite_get_table_printf", test_get_table_printf,
-      0, 0);
-  Tcl_CreateCommand(interp, "sqlite_close", sqlite_test_close, 0, 0);
-  Tcl_CreateCommand(interp, "sqlite_create_function", 
-      sqlite_test_create_function, 0, 0);
-  Tcl_CreateCommand(interp, "sqlite_create_aggregate",
-      sqlite_test_create_aggregate, 0, 0);
-  Tcl_CreateCommand(interp, "sqlite_register_test_function",
-      sqlite_register_test_function, 0, 0);
+  extern int sqlite_open_file_count;
+  static struct {
+     char *zName;
+     Tcl_CmdProc *xProc;
+  } aCmd[] = {
+     { "sqlite_mprintf_int",             (Tcl_CmdProc*)sqlite_mprintf_int    },
+     { "sqlite_mprintf_str",             (Tcl_CmdProc*)sqlite_mprintf_str    },
+     { "sqlite_mprintf_double",          (Tcl_CmdProc*)sqlite_mprintf_double },
+     { "sqlite_open",                    (Tcl_CmdProc*)sqlite_test_open      },
+     { "sqlite_last_insert_rowid",       (Tcl_CmdProc*)test_last_rowid       },
+     { "sqlite_exec_printf",             (Tcl_CmdProc*)test_exec_printf      },
+     { "sqlite_get_table_printf",        (Tcl_CmdProc*)test_get_table_printf },
+     { "sqlite_close",                   (Tcl_CmdProc*)sqlite_test_close     },
+     { "sqlite_create_function",         (Tcl_CmdProc*)test_create_function  },
+     { "sqlite_create_aggregate",        (Tcl_CmdProc*)test_create_aggregate },
+     { "sqlite_register_test_function",  (Tcl_CmdProc*)test_register_func    },
+     { "sqlite_abort",                   (Tcl_CmdProc*)sqlite_abort          },
+     { "sqlite_datatypes",               (Tcl_CmdProc*)sqlite_datatypes      },
+#ifdef MEMORY_DEBUG
+     { "sqlite_malloc_fail",             (Tcl_CmdProc*)sqlite_malloc_fail    },
+     { "sqlite_malloc_stat",             (Tcl_CmdProc*)sqlite_malloc_stat    },
+#endif
+     { "sqlite_compile",                 (Tcl_CmdProc*)test_compile          },
+     { "sqlite_step",                    (Tcl_CmdProc*)test_step             },
+     { "sqlite_finalize",                (Tcl_CmdProc*)test_finalize         },
+     { "breakpoint",                     (Tcl_CmdProc*)test_breakpoint       },
+  };
+  int i;
+
+  for(i=0; i<sizeof(aCmd)/sizeof(aCmd[0]); i++){
+    Tcl_CreateCommand(interp, aCmd[i].zName, aCmd[i].xProc, 0, 0);
+  }
   Tcl_LinkVar(interp, "sqlite_search_count", 
       (char*)&sqlite_search_count, TCL_LINK_INT);
-#ifdef MEMORY_DEBUG
-  Tcl_CreateCommand(interp, "sqlite_malloc_fail", sqlite_malloc_fail, 0, 0);
-  Tcl_CreateCommand(interp, "sqlite_malloc_stat", sqlite_malloc_stat, 0, 0);
-#endif
-  Tcl_CreateCommand(interp, "sqlite_abort", sqlite_abort, 0, 0);
+  Tcl_LinkVar(interp, "sqlite_open_file_count", 
+      (char*)&sqlite_open_file_count, TCL_LINK_INT);
   return TCL_OK;
 }
