@@ -14,7 +14,7 @@
 ** This file contains functions for allocating memory, comparing
 ** strings, and stuff like that.
 **
-** $Id: util.c,v 1.50 2002/08/26 19:55:08 drh Exp $
+** $Id: util.c,v 1.64 2003/06/02 06:17:10 jplyon Exp $
 */
 #include "sqliteInt.h"
 #include <stdarg.h>
@@ -48,7 +48,7 @@ static int memcnt = 0;
 ** Allocate new memory and set it to zero.  Return NULL if
 ** no memory is available.
 */
-void *sqliteMalloc_(int n, char *zFile, int line){
+void *sqliteMalloc_(int n, int bZero, char *zFile, int line){
   void *p;
   int *pi;
   int k;
@@ -76,7 +76,7 @@ void *sqliteMalloc_(int n, char *zFile, int line){
   pi[1] = n;
   pi[k+2] = 0xdead3344;
   p = &pi[2];
-  memset(p, 0, n);
+  memset(p, bZero==0, n);
 #if MEMORY_DEBUG>1
   fprintf(stderr,"%06d malloc %d bytes at 0x%x from %s:%d\n",
       ++memcnt, n, (int)p, zFile,line);
@@ -84,6 +84,24 @@ void *sqliteMalloc_(int n, char *zFile, int line){
   return p;
 }
 
+/*
+** Check to see if the given pointer was obtained from sqliteMalloc()
+** and is able to hold at least N bytes.  Raise an exception if this
+** is not the case.
+**
+** This routine is used for testing purposes only.
+*/
+void sqliteCheckMemory(void *p, int N){
+  int *pi = p;
+  int n, k;
+  pi -= 2;
+  assert( pi[0]==0xdead1122 );
+  n = pi[1];
+  assert( N>=0 && N<n );
+  k = (n+sizeof(int)-1)/sizeof(int);
+  assert( pi[k+2]==0xdead3344 );
+}
+
 /*
 ** Free memory previously obtained from sqliteMalloc()
 */
@@ -121,7 +139,7 @@ void *sqliteRealloc_(void *oldP, int n, char *zFile, int line){
   int *oldPi, *pi, k, oldN, oldK;
   void *p;
   if( oldP==0 ){
-    return sqliteMalloc_(n,zFile,line);
+    return sqliteMalloc_(n,1,zFile,line);
   }
   if( n==0 ){
     sqliteFree_(oldP,zFile,line);
@@ -153,7 +171,7 @@ void *sqliteRealloc_(void *oldP, int n, char *zFile, int line){
   if( n>oldN ){
     memset(&((char*)p)[oldN], 0, n-oldN);
   }
-  memset(oldPi, 0, (oldK+3)*sizeof(int));
+  memset(oldPi, 0xab, (oldK+3)*sizeof(int));
   free(oldPi);
 #if MEMORY_DEBUG>1
   fprintf(stderr,"%06d realloc %d to %d bytes at 0x%x to 0x%x at %s:%d\n",
@@ -190,14 +208,14 @@ void sqliteStrRealloc(char **pz){
 char *sqliteStrDup_(const char *z, char *zFile, int line){
   char *zNew;
   if( z==0 ) return 0;
-  zNew = sqliteMalloc_(strlen(z)+1, zFile, line);
+  zNew = sqliteMalloc_(strlen(z)+1, 0, zFile, line);
   if( zNew ) strcpy(zNew, z);
   return zNew;
 }
 char *sqliteStrNDup_(const char *z, int n, char *zFile, int line){
   char *zNew;
   if( z==0 ) return 0;
-  zNew = sqliteMalloc_(n+1, zFile, line);
+  zNew = sqliteMalloc_(n+1, 0, zFile, line);
   if( zNew ){
     memcpy(zNew, z, n);
     zNew[n] = 0;
@@ -214,10 +232,12 @@ char *sqliteStrNDup_(const char *z, int n, char *zFile, int line){
 
 /*
 ** Allocate new memory and set it to zero.  Return NULL if
-** no memory is available.
+** no memory is available.  See also sqliteMallocRaw().
 */
 void *sqliteMalloc(int n){
-  void *p = malloc(n);
+  void *p;
+  if( n==0 ) return 0;
+  p = malloc(n);
   if( p==0 ){
     sqlite_malloc_failed++;
     return 0;
@@ -226,6 +246,21 @@ void *sqliteMalloc(int n){
   return p;
 }
 
+/*
+** Allocate new memory but do not set it to zero.  Return NULL if
+** no memory is available.  See also sqliteMalloc().
+*/
+void *sqliteMallocRaw(int n){
+  void *p;
+  if( n==0 ) return 0;
+  p = malloc(n);
+  if( p==0 ){
+    sqlite_malloc_failed++;
+    return 0;
+  }
+  return p;
+}
+
 /*
 ** Free memory previously obtained from sqliteMalloc()
 */
@@ -262,14 +297,14 @@ void *sqliteRealloc(void *p, int n){
 char *sqliteStrDup(const char *z){
   char *zNew;
   if( z==0 ) return 0;
-  zNew = sqliteMalloc(strlen(z)+1);
+  zNew = sqliteMallocRaw(strlen(z)+1);
   if( zNew ) strcpy(zNew, z);
   return zNew;
 }
 char *sqliteStrNDup(const char *z, int n){
   char *zNew;
   if( z==0 ) return 0;
-  zNew = sqliteMalloc(n+1);
+  zNew = sqliteMallocRaw(n+1);
   if( zNew ){
     memcpy(zNew, z, n);
     zNew[n] = 0;
@@ -282,7 +317,8 @@ char *sqliteStrNDup(const char *z, int n){
 ** Create a string from the 2nd and subsequent arguments (up to the
 ** first NULL argument), store the string in memory obtained from
 ** sqliteMalloc() and make the pointer indicated by the 1st argument
-** point to that string.
+** point to that string.  The 1st argument must either be NULL or 
+** point to memory obtained from sqliteMalloc().
 */
 void sqliteSetString(char **pz, const char *zFirst, ...){
   va_list ap;
@@ -298,7 +334,7 @@ void sqliteSetString(char **pz, const char *zFirst, ...){
   }
   va_end(ap);
   sqliteFree(*pz);
-  *pz = zResult = sqliteMalloc( nByte );
+  *pz = zResult = sqliteMallocRaw( nByte );
   if( zResult==0 ){
     return;
   }
@@ -320,7 +356,9 @@ void sqliteSetString(char **pz, const char *zFirst, ...){
 /*
 ** Works like sqliteSetString, but each string is now followed by
 ** a length integer which specifies how much of the source string 
-** to copy (in bytes).  -1 means use the whole string.
+** to copy (in bytes).  -1 means use the whole string.  The 1st 
+** argument must either be NULL or point to memory obtained from 
+** sqliteMalloc().
 */
 void sqliteSetNString(char **pz, ...){
   va_list ap;
@@ -339,7 +377,7 @@ void sqliteSetNString(char **pz, ...){
   }
   va_end(ap);
   sqliteFree(*pz);
-  *pz = zResult = sqliteMalloc( nByte + 1 );
+  *pz = zResult = sqliteMallocRaw( nByte + 1 );
   if( zResult==0 ) return;
   va_start(ap, pz);
   while( (z = va_arg(ap, const char*))!=0 ){
@@ -357,6 +395,134 @@ void sqliteSetNString(char **pz, ...){
   va_end(ap);
 }
 
+/*
+** Add an error message to pParse->zErrMsg and increment pParse->nErr.
+** The following formatting characters are allowed:
+**
+**      %s      Insert a string
+**      %z      A string that should be freed after use
+**      %d      Insert an integer
+**      %T      Insert a token
+**      %S      Insert the first element of a SrcList
+*/
+void sqliteErrorMsg(Parse *pParse, const char *zFormat, ...){
+  va_list ap;
+  int nByte;
+  int i, j;
+  char *z;
+  static char zNull[] = "NULL";
+
+  pParse->nErr++;
+  nByte = 1 + strlen(zFormat);
+  va_start(ap, zFormat);
+  for(i=0; zFormat[i]; i++){
+    if( zFormat[i]!='%' || zFormat[i+1]==0 ) continue;
+    i++;
+    switch( zFormat[i] ){
+      case 'd': {
+        (void)va_arg(ap, int);
+        nByte += 20;
+        break;
+      }
+      case 'z':
+      case 's': {
+        char *z2 = va_arg(ap, char*);
+        if( z2==0 ) z2 = zNull;
+        nByte += strlen(z2);
+        break;
+      }
+      case 'T': {
+        Token *p = va_arg(ap, Token*);
+        nByte += p->n;
+        break;
+      }
+      case 'S': {
+        SrcList *p = va_arg(ap, SrcList*);
+        int k = va_arg(ap, int);
+        assert( p->nSrc>k && k>=0 );
+        nByte += strlen(p->a[k].zName);
+        if( p->a[k].zDatabase && p->a[k].zDatabase[0] ){
+          nByte += strlen(p->a[k].zDatabase)+1;
+        }
+        break;
+      }
+      default: {
+        nByte++;
+        break;
+      }
+    }
+  }
+  va_end(ap);
+  z = sqliteMalloc( nByte );
+  if( z==0 ) return;
+  sqliteFree(pParse->zErrMsg);
+  pParse->zErrMsg = z;
+  va_start(ap, zFormat);
+  for(i=j=0; zFormat[i]; i++){
+    if( zFormat[i]!='%' || zFormat[i+1]==0 ) continue;
+    if( i>j ){
+      memcpy(z, &zFormat[j], i-j);
+      z += i-j;
+    }
+    j = i+2;
+    i++;
+    switch( zFormat[i] ){
+      case 'd': {
+        int x = va_arg(ap, int);
+        sprintf(z, "%d", x);
+        z += strlen(z);
+        break;
+      }
+      case 'z':
+      case 's': {
+        int len;
+        char *z2 = va_arg(ap, char*);
+        if( z2==0 ) z2 = zNull;
+        len = strlen(z2);
+        memcpy(z, z2, len);
+        z += len;
+        if( zFormat[i]=='z' && z2!=zNull ){
+          sqliteFree(z2);
+        }
+        break;
+      }
+      case 'T': {
+        Token *p = va_arg(ap, Token*);
+        memcpy(z, p->z, p->n);
+        z += p->n;
+        break;
+      }
+      case 'S': {
+        int len;
+        SrcList *p = va_arg(ap, SrcList*);
+        int k = va_arg(ap, int);
+        assert( p->nSrc>k && k>=0 );
+        if( p->a[k].zDatabase && p->a[k].zDatabase[0] ){
+          len = strlen(p->a[k].zDatabase);
+          memcpy(z, p->a[k].zDatabase, len);
+          z += len;
+          *(z++) = '.';
+        }
+        len = strlen(p->a[k].zName);
+        memcpy(z, p->a[k].zName, len);
+        z += len;
+        break;
+      }
+      default: {
+        *(z++) = zFormat[i];
+        break;
+      }
+    }
+  }
+  va_end(ap);
+  if( i>j ){
+    memcpy(z, &zFormat[j], i-j);
+    z += i-j;
+  }
+  assert( (z - pParse->zErrMsg) < nByte );
+  *z = 0;
+}
+
 /*
 ** Convert an SQL-style quoted string into a normal string by removing
 ** the quote characters.  The conversion is done in-place.  If the
@@ -425,8 +591,7 @@ int sqliteHashNoCase(const char *z, int n){
     h = (h<<3) ^ h ^ UpperToLower[(unsigned char)*z++];
     n--;
   }
-  if( h<0 ) h = -h;
-  return h;
+  return h & 0x7fffffff;
 }
 
 /*
@@ -634,12 +799,12 @@ static int sortStrCmp(const char *atext, const char *btext, int useCase){
 ** Return TRUE if z is a pure numeric string.  Return FALSE if the
 ** string contains any character which is not part of a number.
 **
-** Am empty string is considered numeric.
+** Am empty string is considered non-numeric.
 */
-static int sqliteIsNumber(const char *z){
+int sqliteIsNumber(const char *z){
   if( *z=='-' || *z=='+' ) z++;
   if( !isdigit(*z) ){
-    return *z==0;
+    return 0;
   }
   z++;
   while( isdigit(*z) ){ z++; }
@@ -647,12 +812,12 @@ static int sqliteIsNumber(const char *z){
     z++;
     if( !isdigit(*z) ) return 0;
     while( isdigit(*z) ){ z++; }
-    if( *z=='e' || *z=='E' ){
-      z++;
-      if( *z=='+' || *z=='-' ) z++;
-      if( !isdigit(*z) ) return 0;
-      while( isdigit(*z) ){ z++; }
-    }
+  }
+  if( *z=='e' || *z=='E' ){
+    z++;
+    if( *z=='+' || *z=='-' ) z++;
+    if( !isdigit(*z) ) return 0;
+    while( isdigit(*z) ){ z++; }
   }
   return *z==0;
 }
@@ -745,10 +910,9 @@ int sqliteCompare(const char *atext, const char *btext){
 ** 2.6.3 and earlier.
 */
 int sqliteSortCompare(const char *a, const char *b){
-  int len;
   int res = 0;
   int isNumA, isNumB;
-  int dir;
+  int dir = 0;
 
   while( res==0 && *a && *b ){
     if( a[0]=='N' || b[0]=='N' ){
@@ -797,9 +961,8 @@ int sqliteSortCompare(const char *a, const char *b){
         if( res ) break;
       }
     }
-    len = strlen(&a[1]) + 2;
-    a += len;
-    b += len;
+    a += strlen(&a[1]) + 2;
+    b += strlen(&b[1]) + 2;
   }
   if( dir=='-' || dir=='D' ) res = -res;
   return res;
@@ -993,7 +1156,6 @@ sqliteGlobCompare(const unsigned char *zPattern, const unsigned char *zString){
           zPattern++;
         }
         if( c==0 ) return 1;
-        c = UpperToLower[c];
         if( c=='[' ){
           while( *zString && sqliteGlobCompare(&zPattern[1],zString)==0 ){
             sqliteNextChar(zString);
@@ -1118,16 +1280,23 @@ sqliteLikeCompare(const unsigned char *zPattern, const unsigned char *zString){
 ** But usually the problem will be seen.  The result will be an
 ** error which can be used to debug the application that is
 ** using SQLite incorrectly.
+**
+** Ticket #202:  If db->magic is not a valid open value, take care not
+** to modify the db structure at all.  It could be that db is a stale
+** pointer.  In other words, it could be that there has been a prior
+** call to sqlite_close(db) and db has been deallocated.  And we do
+** not want to write into deallocated memory.
 */
 int sqliteSafetyOn(sqlite *db){
   if( db->magic==SQLITE_MAGIC_OPEN ){
     db->magic = SQLITE_MAGIC_BUSY;
     return 0;
-  }else{
+  }else if( db->magic==SQLITE_MAGIC_BUSY || db->magic==SQLITE_MAGIC_ERROR
+             || db->want_to_close ){
     db->magic = SQLITE_MAGIC_ERROR;
     db->flags |= SQLITE_Interrupt;
-    return 1;
   }
+  return 1;
 }
 
 /*
@@ -1139,11 +1308,12 @@ int sqliteSafetyOff(sqlite *db){
   if( db->magic==SQLITE_MAGIC_BUSY ){
     db->magic = SQLITE_MAGIC_OPEN;
     return 0;
-  }else{
+  }else if( db->magic==SQLITE_MAGIC_OPEN || db->magic==SQLITE_MAGIC_ERROR
+             || db->want_to_close ){
     db->magic = SQLITE_MAGIC_ERROR;
     db->flags |= SQLITE_Interrupt;
-    return 1;
   }
+  return 1;
 }
 
 /*
@@ -1156,7 +1326,7 @@ int sqliteSafetyOff(sqlite *db){
 ** at the wrong time or in the wrong sequence.
 */
 int sqliteSafetyCheck(sqlite *db){
-  if( db->recursionDepth ){
+  if( db->pVdbe!=0 ){
     db->magic = SQLITE_MAGIC_ERROR;
     return 1;
   }
