@@ -11,7 +11,7 @@
 # This file implements some common TCL routines used for regression
 # testing the SQLite library
 #
-# $Id: tester.tcl,v 1.23 2002/06/25 13:16:04 drh Exp $
+# $Id: tester.tcl,v 1.25 2003/03/01 19:45:35 drh Exp $
 
 # Make sure tclsqlite was compiled correctly.  Abort now with an
 # error message if not.
@@ -158,7 +158,7 @@ proc finish_test {} {
   finalize_testing
 }
 proc finalize_testing {} {
-  global nTest nErr nProb
+  global nTest nErr nProb sqlite_open_file_count
   if {$nErr==0} memleak_check
   catch {db close}
   puts "$nErr errors out of $nTest tests"
@@ -167,6 +167,10 @@ proc finalize_testing {} {
     puts "$nProb probabilistic tests also failed, but this does"
     puts "not necessarily indicate a malfunction."
   }
+  if {$sqlite_open_file_count} {
+    puts "$sqlite_open_file_count files were left open"
+    incr nErr
+  }
   exit [expr {$nErr>0}]
 }
 
@@ -210,6 +214,26 @@ proc execsql2 {sql} {
   return $result
 }
 
+# Use the non-callback API to execute multiple SQL statements
+#
+proc stepsql {dbptr sql} {
+  set sql [string trim $sql]
+  set r 0
+  while {[string length $sql]>0} {
+    if {[catch {sqlite_compile $dbptr $sql sqltail} vm]} {
+      return [list 1 $vm]
+    }
+    set sql [string trim $sqltail]
+    while {[sqlite_step $vm N VAL COL]=="SQLITE_ROW"} {
+      foreach v $VAL {lappend r $v}
+    }
+    if {[catch {sqlite_finalize $vm} errmsg]} {
+      return [list 1 $errmsg]
+    }
+  }
+  return $r
+}
+
 # Delete a file or directory
 #
 proc forcedelete {filename} {
