@@ -1,7 +1,7 @@
 #
 # Run this script to generated a faq.html output file
 #
-set rcsid {$Id: faq.tcl,v 1.19 2002/08/24 18:24:57 drh Exp $}
+set rcsid {$Id: faq.tcl,v 1.23 2003/05/29 17:43:08 drh Exp $}
 
 puts {<html>
 <head>
@@ -199,10 +199,15 @@ faq {
 
   <p>The locking mechanism used to control simultaneous access might
   not work correctly if the database file is kept on an NFS filesystem.
+  This is because file locking is broken on some NFS implementations.
   You should avoid putting SQLite database files on NFS if multiple
   processes might try to access the file at the same time.  On Windows,
   Microsoft's documentation says that locking may not work under FAT
-  filesystems if you are not running the Share.exe daemon.</p>
+  filesystems if you are not running the Share.exe daemon.  People who
+  have a lot of experience with Windows tell me that file locking of
+  network files is very buggy and is not dependable.  If what they
+  say is true, sharing an SQLite database between two or more Windows
+  machines might cause unexpected problems.</p>
 
   <p>Locking in SQLite is very course-grained.  SQLite locks the
   entire database.  Big database servers (PostgreSQL, Oracle, etc.)
@@ -311,20 +316,19 @@ ORDER BY name
 faq {
   Are there any known size limits to SQLite databases?
 } {
-  <p>Internally, SQLite can handle databases up to 2^40 bytes (1 terabyte)
-  in size.  But the backend interface to POSIX and Win32 limits files to
-  2^31 (2 gigabytes).</p>
+  <p>As of version 2.7.4, 
+  SQLite can handle databases up to 2^41 bytes (2 terabytes)
+  in size on both Windows and Unix.  Older version of SQLite
+  were limited to databases of 2^31 bytes (2 gigabytes).</p>
 
   <p>SQLite arbitrarily limits the amount of data in one row to 1 megabyte.
   There is a single #define in the source code that can be changed to raise
   this limit as high as 16 megabytes if desired.</p>
 
   <p>There is a theoretical limit of about 2^32 (4 billion) rows
-  in a single table, but there
-  is no way to test this limit without exceeding the maximum file size, so
-  it is not really an issue.  There is also a theoretical limit of about 2^32
-  tables and indices, but again it is not really possible to reach this
-  limit due to the file size constraint.</p>
+  in a single table, but this limit has never been tested.</p>
+  There is also a theoretical limit of about 2^32
+  tables and indices.</p>
 
   <p>The name and "CREATE TABLE" statement for a table must fit entirely
   within a 1-megabyte row of the SQLITE_MASTER table.  Other than this,
@@ -399,6 +403,32 @@ COMMIT;
 </pre></blockquote>
 }
 
+faq {
+  I deleted a lot of data but the database file did not get any
+  smaller.  Is this a bug?
+} {
+  <p>No.  When you delete information from an SQLite database, the
+  unused disk space is added to an internal "free-list" and is reused
+  the next time you insert data.  The disk space is not lost.  But
+  neither is it returned to the operating system.</p>
+
+  <p>If you delete a lot of data and want to shrink the database file,
+  run the VACUUM command (version 2.8.1 and later).  VACUUM will reconstruct
+  the database from scratch.  This will leave the database with an empty
+  free-list and a file that is minimal in size.  Note, however, that the
+  VACUUM can take some time to run (around a half second per megabyte
+  on the Linux box where SQLite is developed) and it can use up to twice
+  as much temporary disk space as the original file while it is running.
+  </p>
+}
+
+faq {
+  Can I use SQLite in my commerical product without paying royalties?
+} {
+  <p>Yes.  SQLite is in the public domain.  No claim of ownership is made
+  to any part of the code.  You can do anything you want with it.</p>
+}
+
 # End of questions and answers.
 #############
 
