@@ -1,7 +1,7 @@
 #
 # Run this TCL script to generate HTML for the index.html file.
 #
-set rcsid {$Id: index.tcl,v 1.67 2002/08/18 19:09:24 drh Exp $}
+set rcsid {$Id: index.tcl,v 1.74 2003/05/17 01:39:40 drh Exp $}
 
 puts {<html>
 <head><title>SQLite: An Embeddable SQL Database Engine</title></head>
@@ -36,8 +36,8 @@ puts {
 Quick Links:
 <ul>
 <li><a href="download.html">Download</a></li>
-<li><a href="http://cvs.hwaci.com:2080/sqlite/timeline">Change Log</a></li>
-<li><a href="http://cvs.hwaci.com:2080/sqlite/tktnew">Report a bug</a></li>
+<li><a href="http://cvs.hwaci.com/sqlite/timeline">Change Log</a></li>
+<li><a href="http://cvs.hwaci.com/sqlite/tktnew">Report a bug</a></li>
 <li><a href="quickstart.html">Quick start</a></li>
 </ul>
 </td></tr></table>
@@ -55,9 +55,10 @@ puts {<h2>Features</h2>
 <li>Atomic commit and rollback protect data integrity.</li>
 <li>Database files can be freely shared between machines with
     different byte orders.</li>
+<li>Supports databases up to 2 terabytes (2^41 bytes) in size.</li>
 <li>Small memory footprint: less than 25K lines of C code.</li>
-<li><a href="speed.html">Four times faster</a> than PostgreSQL.
-    Twice as fast as SQLite 1.0.</li>
+<li><a href="speed.html">Two times faster</a> than PostgreSQL and
+    MySQL for many common operations.</li>
 <li>Very simple 
 <a href="c_interface.html">C/C++ interface</a> requires the use of only
 three functions and one opaque structure.</li>
@@ -76,12 +77,12 @@ puts {<h2>Current Status</h2>
 
 <p>A <a href="changes.html">Change Summary</a> is available on this
 website.  You can also access a detailed
-<a href="http://cvs.hwaci.com:2080/sqlite/timeline">change history</a>,
-<a href="http://cvs.hwaci.com:2080/sqlite/rptview?rn=2">view open bugs</a>,
+<a href="http://cvs.hwaci.com/sqlite/timeline">change history</a>,
+<a href="http://cvs.hwaci.com/sqlite/rptview?rn=2">view open bugs</a>,
 or
-<a href="http://cvs.hwaci.com:2080/sqlite/tktnew">report new bugs</a>
+<a href="http://cvs.hwaci.com/sqlite/tktnew">report new bugs</a>
 at the
-<a href="http://cvs.hwaci.com:2080/sqlite/">CVS server</a>.</p>
+<a href="http://cvs.hwaci.com/sqlite/">CVS server</a>.</p>
 
 <p>Complete source code and precompiled binaries for the latest release are
 <a href="download.html">available for download</a> on this site.  You
@@ -110,22 +111,6 @@ for additional information.
 </p>
 }
 
-puts {<h2>Major Database File Format Change - Version 2.6.0 - 2002 July 17</h2>
-
-<p>Beginning with version 2.6.0, the SQLite database file format changed
-in an incompatible way.  If you open a database file from version 2.5.6
-or earlier with version 2.6.0 or later of the library, then the file format
-will be converted automatically.  This is an irreversible operation.  Once
-the conversion occurs, you will no longer be able to access the database
-file from older versions of the library.  If the database is large, the
-conversion might take some time. (Allow 1 to 2 seconds per megabyte
-of database under Linux.) If the database is read-only,
-the conversion cannot occur and the attempt to open the database will
-fail.
-It is suggested that you make backup copies of older database files
-before attempting to open them with version 2.6.0 or later of the library.</p>
-}
-
 puts {<h2>Documentation</h2>
 
 <p>The following documentation is currently available:</p>
@@ -139,6 +124,8 @@ puts {<h2>Documentation</h2>
 <li>SQLite is <a href="datatypes.html">typeless</a>.
 <li>The <a href="lang.html">SQL Language</a> subset understood by SQLite.</li>
 <li>The <a href="c_interface.html">C/C++ Interface</a>.</li>
+<li>The <a href="nulls.html">NULL handling</a> in SQLite versus
+    other SQL database engines.</li>
 <li>The <a href="tclsqlite.html">Tcl Binding</a> to SQLite.</li>
 <li>The <a href="arch.html">Architecture of the SQLite Library</a> describes
     how the library is put together.</li>
@@ -148,11 +135,14 @@ puts {<h2>Documentation</h2>
     <a href="fileformat.html">database file format</a> used by SQLite.
 <li>A <a href="speed.html">speed comparison</a> between SQLite, PostgreSQL,
     and MySQL.</li>
+<li>User-written documentation is available on the
+    <a href="http://cvs.hwaci.com/sqlite/wiki">SQLite Wiki</a>.  Please
+    contribute if you can.</li>
 </ul>
 </p>
 
 <p>The SQLite source code is 30% comment.  These comments are
-another important source of information. </p>
+another important source of information.  </p>
 
 }
 
@@ -177,7 +167,9 @@ puts {
 If you would like professional support for SQLite
 or if you want custom modifications to SQLite preformed by the
 original author, these services are available for a modest fee.
-For additional information contact:</p>
+For additional information visit
+<a href="http://www.hwaci.com/sw/sqlite/support.html">
+http://www.hwaci.com/sw/sqlite/support.html</a> or contact:</p>
 
 <blockquote>
 D. Richard Hipp <br />
@@ -208,7 +200,7 @@ you desire to the file "Makefile.linux-gcc" and then executing that
 makefile.  Tha latter method is used for all official development
 and testing of SQLite and for building the precompiled
 binaries found on this website.  Windows binaries are generated by
-cross-compiling from Linux using <a href="www.mingw.org">MinGW</a></p>
+cross-compiling from Linux using <a href="http://www.mingw.org/">MinGW</a></p>
 }
 
 puts {<h2>Related Sites</h2>
@@ -220,8 +212,8 @@ using SQLite, visit the Wiki documentation at:
 </p>
 
 <blockquote>
-<a href="http://cvs.hwaci.com:2080/sqlite/wiki">
-http://cvs.hwaci.com:2080/sqlite/wiki</a>
+<a href="http://cvs.hwaci.com/sqlite/wiki">
+http://cvs.hwaci.com/sqlite/wiki</a>
 </blockquote>
 }
 
